#!/bin/sh
# 
# Execute this script to generate a configure script
#
# This file is free software: you are free to change and redistribute it.
# There is NO WARRANTY, to the extent permitted by law.
#
majorversion=0.2

if [ -d ".bzr" ]; then 
 if [ "`which bzr`" = "" ]; then
  echo "this tree uses bazaar but bzr is not installed" 
  exit 1
 fi
 vcs=bzr
elif [ -d ".hg" ]; then
 if [ "`which hg`" = "" ]; then
  echo "this tree uses mercurial but hg is not installed" 
  exit 1
 fi
 vcs=hg
fi

echo -n "cleaning... "
sh autoclean.sh
echo "done"

# wasora version
if [ ${vcs} = "bzr" ]; then
 echo -n "getting bzr revision id... "
 version="${majorversion}.`bzr version-info --custom --template="{revno}"`"
 bzr version-info --custom --template="\
#define WASORA_VCS_BRANCH    \"{branch_nick}\"\n\
#define WASORA_VCS_REVID     \"{revision_id}\"\n\
#define WASORA_VCS_REVNO     {revno}\n\
#define WASORA_VCS_DATE      \"{date}\"\n\
#define WASORA_VCS_CLEAN     {clean}\n" > src/versionvcs.h
 echo "done"
 echo -n "building changelog... "
 bzr log --gnu-changelog > ChangeLog
 echo "done"

elif [ ${vcs} = "hg" ]; then
 echo -n "getting hg revision id... "
 version="${majorversion}.`hg log -r tip --template="{rev}"`"
 hg log -r tip --template="\
#define WASORA_VCS_BRANCH    \"{branch}\"\n\
#define WASORA_VCS_REVID     \"{node}\"\n\
#define WASORA_VCS_REVNO     {rev}\n\
#define WASORA_VCS_DATE      \"{date|isodate}\"\n" > src/versionvcs.h
# clean tag?
 echo "done"
 echo -n "building changelog... "
 hg log --style changelog > ChangeLog
 echo "done"

else
 version=${majorversion}
 touch ChangeLog
fi


echo -n "formatting readme & install... "
if [ ! -z "`which pandoc`" ]; then
 pandoc README.txt -o README
 pandoc README.txt -o README.pdf
 pandoc INSTALL.txt -o INSTALL
 pandoc INSTALL.txt -o INSTALL.pdf
else
 fmt -s README.txt > README
 fmt -s INSTALL.txt > INSTALL
fi
echo "done"


# configure.ac
echo -n "building configure.ac"
cat << EOF > configure.ac
dnl Process this file with autoconf to produce a configure script.
dnl
dnl This file is free software; as a special exception the author gives
dnl unlimited permission to copy and/or distribute it, with or without
dnl modifications, as long as this notice is preserved.
dnl
dnl This program is distributed in the hope that it will be useful, but
dnl WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
dnl implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

AC_INIT([wasora],[${version}],[jeremy@talador.com.ar])

AC_CONFIG_SRCDIR([src/wasora.c])
AC_CONFIG_MACRO_DIR([m4])

AC_CANONICAL_HOST

AM_INIT_AUTOMAKE

AC_PROG_AWK
AC_PROG_CC
AC_PROG_INSTALL
AC_PROG_MAKE_SET
AM_PROG_CC_C_O

WASORA_OPT_FLAGS_C

WASORA_CHECK_BASIC_HEADERS
WASORA_CHECK_BASIC_LIBS
WASORA_CHECK_IDA
WASORA_CHECK_READLINE
WASORA_CHECK_GSL

# tested host_oses are:
#  linux-gnu
#  mingw32msvc (cross compiling from gnu/linux)
#  cygwin
#  mingw32
AS_CASE([\${host_os}],
  [linux*],  [os_family="linux"],  dnl I know the OS is not linux but gnu, but this refers to the way the kernel works
  [mingw*],  [os_family="mingw"],
  [cygwin*], [os_family="cygwin"],
  [os_family="unknown"]
)
AM_CONDITIONAL([MINGW], [test "x\${os_family}" == "xmingw"])

AC_CONFIG_FILES([Makefile src/Makefile]) 

WASORA_HOST_VERSION_H

AC_OUTPUT
echo
AS_BOX([Configuration summary])
WASORA_RESUME_LIBS
EOF
echo "done"

am="src/Makefile.am"
echo -n "building $am... "
cat << EOF > $am
AUTOMAKE_OPTIONS = subdir-objects
ACLOCAL_AMFLAGS = \$(ACLOCAL_FLAGS)

bin_PROGRAMS = wasora

wasora_SOURCES = \\
EOF
cd src
find . -maxdepth 2 \( -name "*.c" -o -name "*.h" \) | xargs >> ../$am
cd ..

cat << EOF >> $am

wasora_INCLUDES = \$(all_includes)
wasora_LDADD = \$(WASORALIBS) \$(all_libraries) 
if MINGW
 wasora_LDFLAGS = -static 
else
 wasora_LDFLAGS = -rdynamic
endif
EOF

echo "done"

echo "calling autoreconf... "
autoreconf -i
echo "done"
