import re

from pygments.lexer import RegexLexer, include, bygroups, using, \
     this, combined, ExtendedRegexLexer
from pygments.token import Error, Punctuation, Literal, Token, \
     Text, Comment, Operator, Keyword, Name, String, Number, Generic

__all__ = ['WasoraLexer']

class WasoraLexer(RegexLexer):
    """
    Lexer for wasora input files.

    *Not in pygments official distribution
    """
    name = 'wasora'
    aliases = ['wasora']
    filenames = ['*.was']
    mimetypes = ['text/plain']

    tokens = {
        'root': [
            # Keywords
             (r'\b(INCLUDE|FROM|TO|ABORT|DEFAULT_ARGUMENT_VALUE|IMPLICIT|TIME_PATH|INITIAL_CONDITIONS_MODE|LOAD_PLUGIN|LOAD_ROUTINE|NUMBER|VAR|CONST|ALIAS|VECTOR|SIZE|DATA|FUNCTION_DATA|MATRIX|ROWS|COLS|FUNCTION|FILE_PATH|ROUTINE|VECTOR_DATA|VECTORS|COLUMNS|INTERPOLATION_THRESHOLD|INTERPOLATION|FILE|OUTPUT_FILE|INPUT_FILE|MODE|INPUT|OUTPUT|OPEN|DO_NOT_OPEN|CLOSE|IF|ELSE|ENDIF|SEMAPHORE|SEM|READ|WRITE|SHM|SHM_OBJECT|ASCII_FILE_PATH|BINARY_FILE_PATH|ASCII_FILE|BINARY_FILE|IGNORE_NULL|PRINT|NONEWLINE|SEP|SEPARATOR|NOSEP|HEADER|STRING|TEXT|PRINT_FUNCTION|MIN|MAX|STEP|FORMAT|PRINT_VECTOR|VERTICAL|HORIZONTAL|ELEMS_PER_LINE|TEMPLATE|WILDCARD|SHELL|CALL|HISTORY|PARAMETRIC|TYPE|NSTEPS|OUTER_STEPS|OFFSET|FIT|VIA|GRADIENT|RANGE_MIN|RANGE_MAX|DELTAEPSREL|DELTAEPSABS|MAX_ITER|VERBOSE|DO_NOT_RERUN|NORERUN|RERUN|OPTIMIZE|MINIMIZE|SIMAN_EFUNC|METHOD|ALGORITHM|GUESS|SIMAN_INIT|SIMAN_STEP|SIMAN_COPY|SIMAN_COPY_CONSTRUCT|SIMAN_PRINT|SIMAN_BEST|SIMAN_DESTROY|PHASE_SPACE|DIFFERENTIAL|NONE|ALLOWED|AS_PROVIDED|FROM_VARIABLES|FROM_DERIVATIVES|WAIT|POST|SKIP_STEP|SKIP_STATIC_STEP|SKIP_TIME|SKIP_HEADER_STEP|MAX_ITER|TOL|GRADTOL|TRIES|K|T_INIT|T_MIN|MU)\s*\b',
             Keyword),

            # Reserved variables
             (r'\b(done|done_outer|done_static|done_transient|dt|end_time|error|computing|initial|conditions|from|variable|values,|error|=|%d|i|in_outer_initial|in_static|in_static_first|in_static_last|in_transient|in_transient_first|in_transient_last|infinite|j|max_daughters|max_dt|min_dt|on_gsl_error|on_ida_error|on_nan|pi|realtime_scale|rel_error|static_steps|step_inner|step_outer|step_static|step_transient|t|zero)\s*\b',
             Name.Builtin),

            # Built-in functions, functionals and vectorfunctions
             (r'\b(abs|acos|asin|atan|atan2|builtindecl.h|ceil|clock|cos|cosh|d_dt|deadband|derivative|equal|exp|floor|func_min|gauss_kronrod|gauss_legendre|heaviside|if|integral|integral_dt|integral_euler_dt|j0|lag|lag_bilinear|lag_euler|last|limit|limit_dt|log|mark_max|mark_min|max|min|mod|not|prod|random|random_gauss|root|round|sawtooth_wave|sgn|sin|sinh|sqrt|square_wave|sum|tan|tanh|threshold_max|threshold_min|triangular_wave|vecdot|vecmax|vecmaxindex|vecmin|vecminindex|vecnorm|vecsize|vecsum)\s*\b',
             Name.Function),

            # Numbers
            (r'([0-9]*\.[0-9]*|[0-9]+)(e[+-]?[0-9]+)?', Number),
            (r' .[0-9]+', Number),

            # Operators
            (r'[&\|=!<>\+-\/\*\^]', Operator),

            # Punctuation
            (r'[\(\)\[\]\{\}$,:;%\\"\']', Punctuation),

            # Comment (but not if escaped)
            (r'^#.*?\n', Comment),
            (r'\#', Text),
            (r'\\#', Text),
            (r'\\\#', Text),
            (r'.#.*?\n', Comment),

            (r'[a-z][a-z0-9_]*', Text),
            (r'[A-Z][A-Z0-9_]*', Text),
            (r' ', Text),
            (r' .*', Text),

        ],

    }

