#include <stdio.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include <math.h>

#include <gsl/gsl_types.h>
#include <gsl/gsl_rng.h>

#include <interface.h>

#define MAX   21

double *seed;
double *decks;
double *min_bet;
double *max_bet;
double *penetration;
double *ignorebj;
double *ignoresoft;
double *dealerstandsonsoft17;
double *bjratio;
double *hands;
double *hand;

double *dealt_card;
double dealer_hidden_card;

double *dealer_busts;
double *dealer_bjs;

double *dealer_state;
double *dealer_count;
gsl_vector *dealer_hand;
double *dealer_current_card;

double *player_says;
double *player_bankroll;
double *player_wins;
double *player_loses;
double *player_pushes;
double *player_busts;
double *player_bjs;

gsl_vector *player_bet;
gsl_vector *player_count;
gsl_matrix *player_hand;
double *player_current_hand;
double *player_current_card;

double *dealer_messages;

double *done_static;

gsl_vector *shoe;
char suitname[][12] = {"none", "spades", "hearts", "diamonds", "clubs"};
char numbername[][4] = {"0", "A", "2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K"};

#define card_value(card)  ((int)(card) % 100) 
#define card_number(card) (((int)card) / 1000)
#define card_suit(card)   ((((int)card) / 100) % 10)

char *infotxt[29];

#define ACTION_NONE                                0
#define ACTION_NEW_SHOE_SHUFFLING                  1 
#define ACTION_ASKING_BET                          2
#define ACTION_ACCEPTING_BET                       3
#define ACTION_DEALING_FIRST_CARD_TO_PLAYER        4
#define ACTION_DEALING_FIRST_CARD_TO_SELF          5
#define ACTION_DEALING_SECOND_CARD_TO_PLAYER       6
#define ACTION_CHECKING_PLAYER_BJ                  7
#define ACTION_WAITING_PLAYER_DECISION             8
#define ACTION_HAND_IS_OVER                        9
#define ACTION_GAME_OVER                          10

#define PLAYER_HITS                               12
#define PLAYER_STAYS                              11
#define PLAYER_BUSTS                              13
#define PLAYER_HAS_BLACKJACK                      14
#define PLAYER_SPLITS                             16
#define PLAYER_DOUBLES_DOWN                       17
#define PLAYER_SURRENDERS                         18

#define DEALER_HITS                               19
#define DEALER_STAYS                              20
#define DEALER_BUSTS                              21
#define DEALER_HAS_BLACKJACK                      22
#define DEALER_DOES_NOT_HAVE_BLACKJACK            23

#define NOTHING      0
#define STAY         1
#define HIT          2
#define DOUBLE_DOWN  3



void dealer_text_message(int message, int card, int count) {
  
  int player;
  int nhand;
  int ncard;
  int info;
  
  if (*(dealer_messages) == 0) {
    return;
  }
  
  player = message / 10000;
  nhand = (message / 1000) % 10;
  ncard = (message / 100) % 10;
  info = message % 100;
  
  printf("dealer: ");
  printf("%s ", infotxt[info]);
  
  if (nhand != 0) {
    printf("hand #%d ", nhand);
  }
  
  if (ncard != 0) {
    printf("card #%d ", ncard);
  }
  
  if (card != 0) {
    printf("[%s of %s] ", numbername[card_number(card)], suitname[card_suit(card)]);
  }

  if (count != 0) {
    printf("count is %s %d ", (count<0)?"soft":"hard", abs(count));
  }

  printf("\n");
  
  return;
  
}

int compute_count(double *hand) {

  int i;
  int cards = 0;
  int aces = 0;
  int count = 0;

  for (i = 0; i < MAX; i++) {
    count += card_value(hand[i]);
    cards += ((int)hand[i] != 0);
    if (card_value(hand[i]) == 11) {
      aces++;
    }
    if (count > 21 && aces > 0) {
      count -= 10;
      aces--;
    }
  }
  if (cards > 1 && aces > 0) {
    count = -count;
  }

  return count;
  
}


void shuffle(double *arg) {

  int i, j;
  int deck, suit, number;
  static gsl_rng *r;

  if (shoe == NULL) {
    shoe = gsl_vector_alloc((int)(*(decks))*52);
//    r = gsl_rng_alloc(gsl_rng_knuthran2002);
    r = gsl_rng_alloc(gsl_rng_mt19937);

    // inicializamos el rng al principio, despues le seguimos pidiendo
    // randoms sin inicializar sino salen siempre los mismos shoes
    gsl_rng_set(r, (int)*(seed));
  }

  // inicializamos el deck de nuevo porque sino vamos agregando
  // cartas ficticiamente
  i = 0;
  for (deck = 0; deck < (int)(*(decks)); deck++) {
    for (suit = 1; suit < 5; suit++) {

      // ace s
      gsl_vector_set(shoe, i++, 1000 + 100*suit + 11);

      // numeros
      for (number = 2; number <= 10; number++) {
        gsl_vector_set(shoe, i++, 1000*number + 100*suit + number);
      }

      // jacks
      gsl_vector_set(shoe, i++, 11000 + 100*suit + 10);

      // queens
      gsl_vector_set(shoe, i++, 12000 + 100*suit + 10);

      // king
      gsl_vector_set(shoe, i++, 13000 + 100*suit + 10);
    }
  }  
   
  for (i = 52*(int)(*(decks))-1; i > 0; i--) {
    j = gsl_rng_uniform_int(r, i);
    gsl_vector_swap_elements(shoe, i, j);
  }

  // si nos pidieron que dieramos ciertas cartas en particular
  if (arg != NULL) {
   
    // recordar que la cero se quema!
    // para el dealer, carta 2
    if (arg[0] != 0) {
      gsl_vector_set(shoe, 2,  (arg[0]==11?1:arg[0])*1000 + 100*3 + arg[0]);
    }

    // para el dealer, carta 4
    if (arg[1] != 0) {
      gsl_vector_set(shoe, 4,  (arg[1]==11?1:arg[1])*1000 + 100*3 + arg[1]);
    }

    // para el jugador carta 1    
    if (arg[2] != 0) {
      gsl_vector_set(shoe, 1,  (arg[2]==11?1:arg[2])*1000 + 100*3 + arg[2]);
    }
    // para el jugador carta 3
    if (arg[3] != 0) {
      gsl_vector_set(shoe, 3,  (arg[3]==11?1:arg[3])*1000 + 100*3 + arg[3]);
    }
    
    
  }
    
  return;
}


void print_card(double card) {
  printf("%s of %s ", numbername[card_number(card)], suitname[card_suit(card)]);
  return;
}


double dealer(double *arg) {
  
  static int shoe_current_card;
  
  if (dealer_state == NULL || (int)(wasora_get_var_value("step_static")) == 1) {
    assert((seed = wasora_get_var_value_ptr("seed")) != NULL);

    assert((decks = wasora_get_var_value_ptr("decks")) != NULL);
    
    assert((min_bet = wasora_get_var_value_ptr("min_bet")) != NULL);
    assert((max_bet = wasora_get_var_value_ptr("max_bet")) != NULL);
    assert((penetration = wasora_get_var_value_ptr("penetration")) != NULL);
    assert((ignorebj = wasora_get_var_value_ptr("ignorebj")) != NULL);
    assert((ignoresoft = wasora_get_var_value_ptr("ignoresoft")) != NULL);
    assert((dealerstandsonsoft17 = wasora_get_var_value_ptr("dealerstandsonsoft17")) != NULL);
    assert((bjratio = wasora_get_var_value_ptr("bjratio")) != NULL);
    assert((hand = wasora_get_var_value_ptr("hand")) != NULL);
    assert((hands = wasora_get_var_value_ptr("hands")) != NULL);

    assert((dealt_card = wasora_get_var_value_ptr("dealt_card")) != NULL);

    assert((dealer_busts = wasora_get_var_value_ptr("dealer_busts")) != NULL);
    assert((dealer_bjs = wasora_get_var_value_ptr("dealer_bjs")) != NULL);

    assert((dealer_state = wasora_get_var_value_ptr("dealer_state")) != NULL);
    assert((dealer_count = wasora_get_var_value_ptr("dealer_count")) != NULL);
    assert((dealer_hand = wasora_get_vector_gsl_ptr("dealer_hand")) != NULL);
    assert((dealer_current_card = wasora_get_var_value_ptr("dealer_current_card")) != NULL);

    assert((player_bankroll = wasora_get_var_value_ptr("player_bankroll")) != NULL);
    assert((player_wins = wasora_get_var_value_ptr("player_wins")) != NULL);
    assert((player_loses = wasora_get_var_value_ptr("player_loses")) != NULL);
    assert((player_pushes = wasora_get_var_value_ptr("player_pushes")) != NULL);
    assert((player_busts = wasora_get_var_value_ptr("player_busts")) != NULL);
    assert((player_bjs = wasora_get_var_value_ptr("player_bjs")) != NULL);
    
    assert((player_bet = wasora_get_vector_gsl_ptr("player_bet")) != NULL);
    assert((player_says = wasora_get_var_value_ptr("player_says")) != NULL);
    assert((player_count = wasora_get_vector_gsl_ptr("player_count")) != NULL);
    assert((player_hand = wasora_get_matrix_gsl_ptr("player_hand")) != NULL);
    assert((player_current_hand = wasora_get_var_value_ptr("player_current_hand")) != NULL);
    assert((player_current_card = wasora_get_var_value_ptr("player_current_card")) != NULL);
    
    assert((dealer_messages = wasora_get_var_value_ptr("dealer_messages")) != NULL);
    assert((done_static = wasora_get_var_value_ptr("done_static")) != NULL);
    
    infotxt[ACTION_NONE] = "ok";
    infotxt[ACTION_NEW_SHOE_SHUFFLING] = "new shoe, i'm shuffling";
    infotxt[ACTION_ASKING_BET] = "please place bet";
    infotxt[ACTION_ACCEPTING_BET] = "bet accepted";
    infotxt[ACTION_DEALING_FIRST_CARD_TO_PLAYER] = "dealing first card to player";
    infotxt[ACTION_DEALING_FIRST_CARD_TO_SELF] = "dealing to dealer";
    infotxt[ACTION_DEALING_SECOND_CARD_TO_PLAYER] = "dealing second card to player";
    infotxt[ACTION_CHECKING_PLAYER_BJ] = "checking if player has blackjack";
    infotxt[ACTION_WAITING_PLAYER_DECISION] = "waiting player decision";
    infotxt[ACTION_HAND_IS_OVER] = "hand is over, collect your chips";
    infotxt[ACTION_GAME_OVER] = "game is over folks! good night";

    infotxt[PLAYER_HITS] = "player hits";
    infotxt[PLAYER_STAYS] = "player stays, moving to next hand";
    infotxt[PLAYER_BUSTS] = "player busts, moving to next hand";
    infotxt[PLAYER_HAS_BLACKJACK] = "player has blackjack, moving to next hand";
    infotxt[PLAYER_SPLITS] = "player splits hits hand";
    infotxt[PLAYER_DOUBLES_DOWN] = "player doubles down";
    infotxt[PLAYER_SURRENDERS] = "player surrenders, moving to next hand";
    
    infotxt[DEALER_HITS] = "dealer hits";
    infotxt[DEALER_STAYS] = "dealer stays";
    infotxt[DEALER_BUSTS] = "dealer busts";
    infotxt[DEALER_HAS_BLACKJACK] = "dealer has blackjack";

    shoe_current_card = (int)*(decks)*52; // to make sure we shuffle in the first hand
    *(hand) = 0;    
    *(dealer_state) = ACTION_HAND_IS_OVER;
  }

/*  
  // si nos pidieron que ignoremos las manos suaves, si despues del paso 4
  // la cuenta es negativa entonces ignoramos la mano
  if ((int)*(ignoresoft) != 0 && step > 4 &&
      gsl_vector_get(player_count, 0) < 0) {
//    printf("%d\n", (int)gsl_vector_get(player_count->current, 0));
    *(hand) -= 1;
    hand_is_over = 1;
  }
*/  
  // forget previously dealt card
  (*dealt_card) = 0;
  
// -----------------------------------------------------------------  
  if ((int)(*dealer_state) == ACTION_HAND_IS_OVER) {
    // check if we are done
    if ((int)(*hand) >= (int)(*hands)) {
      *(done_static) = 1;
      
      // dealer says goodbye
      *(dealer_state) = ACTION_GAME_OVER;
      dealer_text_message((int)(*dealer_state), 0, 0);
      
    // if not, start a new hand  
    } else {
    
      // all hands are emptied
      gsl_matrix_set_zero(player_hand);
      gsl_vector_set_zero(dealer_hand);
      (*player_current_hand) = 1;
      (*player_current_card) = 0;
      (*dealer_current_card) = 0;
      *(hand) += 1;

      // but check if the shoe has to be shuffled
      if ((double)shoe_current_card/((*decks)*52) > *(penetration)) {
        // penetration has been attained, dealer shuffles the shoe
        shuffle(arg);
    
        // TODO: asks players to cut
        // TODO: tell the players we are shuffling
        // dealer burns the first card (and does not show it)
        shoe_current_card = 1;
      
      }

      // dealer asks for player one's bet
      *(dealer_state) = (double)(ACTION_ASKING_BET);
      dealer_text_message((int)(*dealer_state), 0, 0);
    }

// -----------------------------------------------------------------        
  } else if ((int)(*dealer_state) == ACTION_ASKING_BET) {
    double bet = *(player_says);
    
    // dealer checks bets from player one
    // TODO: check for drinking milk
/*  
    if (bet > *(max_bet)) {
      bet = *(max_bet);
    }
    
    if (*(player_bankroll) < bet) {
      bet = *(player_bankroll);
    }
*/
    gsl_vector_set(player_bet, 0, bet);

    *(dealer_state) = ACTION_ACCEPTING_BET;
    dealer_text_message((int)(*dealer_state), 0, 0);

// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == ACTION_ACCEPTING_BET) {
    // player first card
    
    // dealer deals next card
    (*dealt_card) = gsl_vector_get(shoe, shoe_current_card++);
    
    // dealer gives the dealt card to the player
    gsl_matrix_set(player_hand, (int)((*player_current_hand)-1), (int)(++(*player_current_card))-1, (*dealt_card));

    // dealer sets the player's count
    gsl_vector_set(player_count, (int)((*player_current_hand)-1), (double)compute_count(gsl_matrix_ptr(player_hand, 0, 0)));
    
    *(dealer_state) = ACTION_DEALING_FIRST_CARD_TO_PLAYER;
    dealer_text_message((int)(*dealer_state), (int)(*dealt_card), (int)gsl_vector_get(player_count, 0));

// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == ACTION_DEALING_FIRST_CARD_TO_PLAYER) {
    // dealer first card
    
    // dealer deals next card
    (*dealt_card) = gsl_vector_get(shoe, shoe_current_card++);

    // dealer gives the dealt card to himself
    gsl_vector_set(dealer_hand, (int)(++(*dealer_current_card))-1, (*dealt_card));

    // dealer sets his own count
    *(dealer_count) = (double)compute_count(gsl_vector_ptr(dealer_hand, 0));
    
    // dealer shows his card
    *(dealer_state) = ACTION_DEALING_FIRST_CARD_TO_SELF;
    dealer_text_message((int)(*dealer_state), (int)(*dealt_card), (int)*(dealer_count));

// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == ACTION_DEALING_FIRST_CARD_TO_SELF) {
    // player second card
    
    // dealer deals next card
    (*dealt_card) = gsl_vector_get(shoe, shoe_current_card++);
    
    // dealer gives the dealt card to the player
    gsl_matrix_set(player_hand, (int)((*player_current_hand)-1), (int)(++(*player_current_card))-1, (*dealt_card));

    // dealer sets the player's count
    gsl_vector_set(player_count, (int)((*player_current_hand)-1), (double)compute_count(gsl_matrix_ptr(player_hand, 0, 0)));
    
    *(dealer_state) = ACTION_DEALING_SECOND_CARD_TO_PLAYER;
    dealer_text_message((int)(*dealer_state), (int)(*dealt_card), (int)gsl_vector_get(player_count, 0));

// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == ACTION_DEALING_SECOND_CARD_TO_PLAYER) {
    // ask the player to play
    
    *(dealer_state) = ACTION_CHECKING_PLAYER_BJ;
    dealer_text_message((int)(*dealer_state), 0, 0);
    
// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == ACTION_CHECKING_PLAYER_BJ) {
    
    if ((int)gsl_vector_get(player_count, 0) == -21) {
      *(dealer_state) = PLAYER_HAS_BLACKJACK;
      dealer_text_message((int)(*dealer_state), (int)(*dealt_card), 0);
    } else {
      *(dealer_state) = ACTION_WAITING_PLAYER_DECISION;
      dealer_text_message((int)(*dealer_state), 0, 0);
    }

// -----------------------------------------------------------------        
  } else if ((int)(*dealer_state) == ACTION_WAITING_PLAYER_DECISION || (int)(*dealer_state == PLAYER_HITS)) {
    // process the player play
    if ((int)(*player_says) == STAY) {
      // player stays
      gsl_vector_set(player_count, 0, (int)fabs(gsl_vector_get(player_count, 0)));

      // so we now give the turn to the dealer
      *(dealer_state) = PLAYER_STAYS;
      dealer_text_message((int)(*dealer_state), 0, (int)gsl_vector_get(player_count, 0));

// -----------------------------------------------------------------      
    } else if ((int)(*player_says) == HIT) {
      // player hits

      // dealer deals next card
      (*dealt_card) = gsl_vector_get(shoe, shoe_current_card++);

      // dealer gives the dealt card to the player
      gsl_matrix_set(player_hand, (int)((*player_current_hand)-1), (int)(++(*player_current_card))-1, (*dealt_card));

      // dealer sets the player's count
      gsl_vector_set(player_count, (int)((*player_current_hand)-1), (double)compute_count(gsl_matrix_ptr(player_hand, 0, 0)));

      if ((int)fabs(gsl_vector_get(player_count, 0)) > 21) {
        // player busts
        *(dealer_state) = PLAYER_BUSTS;
        dealer_text_message((int)(*dealer_state), (int)(*dealt_card), (int)gsl_vector_get(player_count, 0));

      } else {
        // the player does not bust
        // so dealer asks him to play
        *(dealer_state) = ACTION_WAITING_PLAYER_DECISION;
        dealer_text_message((int)(*dealer_state), (int)(*dealt_card), (int)gsl_vector_get(player_count, 0));
      }
            
    } else {
      // TODO: hacer una cuenta y si el chabon le toma la leche al gato
      // salir quejandonos
      wasora_push_error_message("player_says = %g", *player_says);
      wasora_runtime_error();
              
      *(dealer_state) = ACTION_WAITING_PLAYER_DECISION;
      dealer_text_message((int)(*dealer_state), 0, 0);
    }
      
// TODO: double down, split, insurance, surrender

// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == PLAYER_HAS_BLACKJACK) {
   
    if ((*ignorebj) == 1) {
      // si nos pidieron ignorar blackjacks no contamos la mano
      *(hand) -= 1;
      *(dealer_state) = ACTION_HAND_IS_OVER;
      dealer_text_message((int)(*dealer_state), 0, 0);
      
    } else {
      
      // dealer deals next card
      (*dealt_card) = gsl_vector_get(shoe, shoe_current_card++);

      // dealer gives the dealt card to himself
      gsl_vector_set(dealer_hand, (int)(++(*dealer_current_card))-1, (*dealt_card));

      // dealer sets his own count
      *(dealer_count) = (double)compute_count(gsl_vector_ptr(dealer_hand, 0));
    
      if ((int)fabs((*dealer_count)) == 21) {
        *(dealer_state) = DEALER_HAS_BLACKJACK;
      } else {
        *(dealer_state) = DEALER_DOES_NOT_HAVE_BLACKJACK;
      }      
      dealer_text_message((int)(*dealer_state), (int)(*dealt_card), (int)*(dealer_count));
    }

// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == PLAYER_BUSTS) {
    
    // dealer takes away his money
    *(player_bankroll) -= gsl_vector_get(player_bet, 0);
    *(player_loses) += 1;
    *(player_busts) += 1;
    
    *(dealer_state) = ACTION_HAND_IS_OVER;
    dealer_text_message((int)(*dealer_state), 0, 0);
      
// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == PLAYER_STAYS || (int)(*dealer_state) == DEALER_HITS) {
    
    // dealer deals next card
    (*dealt_card) = gsl_vector_get(shoe, shoe_current_card++);

    // dealer gives the dealt card to himself
    gsl_vector_set(dealer_hand, (int)(++(*dealer_current_card))-1, (*dealt_card));

    // dealer sets his own count
    *(dealer_count) = (double)compute_count(gsl_vector_ptr(dealer_hand, 0));
    
    if ((int)(fabs(*dealer_count)) < 17 || ((int)(*dealerstandsonsoft17) == 0 && (int)(*dealer_count) == -17)) {
      *(dealer_state) = DEALER_HITS;
    } else if ((int)(fabs(*dealer_count)) < 22) {
      *(dealer_state) = DEALER_STAYS;
    } else {
      *(dealer_state) = DEALER_BUSTS;
    }

    dealer_text_message((int)(*dealer_state), (int)(*dealt_card), (int)*(dealer_count));
  
// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == DEALER_STAYS) {
        
    if ((abs((int)gsl_vector_get(player_count, 0)) < 22)) {
      // if player did not bust we check who wins
      // because if he busted, we already took his money away
      if (abs((int)gsl_vector_get(player_count, 0)) < abs((int)*(dealer_count))) {
        *(player_bankroll) -= gsl_vector_get(player_bet, 0);
        *(player_loses) += 1;
      } else if (abs((int)gsl_vector_get(player_count, 0)) > abs((int)*(dealer_count))) {
        *(player_bankroll) += gsl_vector_get(player_bet, 0);
        *(player_wins) += 1;
      } else {
        *(player_pushes) += 1;
      }
    }
        
    *(dealer_state) = ACTION_HAND_IS_OVER;
    dealer_text_message((int)(*dealer_state), 0, 0);

// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == DEALER_BUSTS) {
        
    // everybody who did not bust wins
    // (in single-player game, if we are here then the player did not bust
    // but we check it anyway)
    if ((abs((int)gsl_vector_get(player_count, 0)) < 22)) {
      *(player_bankroll) += gsl_vector_get(player_bet, 0);
      *(player_wins) += 1;
    }
    *(dealer_busts) += 1;
        
    *(dealer_state) = ACTION_HAND_IS_OVER;
    dealer_text_message((int)(*dealer_state), 0, 0);
    
// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == DEALER_HAS_BLACKJACK) {
    
    // if we are here then there is a push with two bjs
    *(player_bjs) += 1;
    *(dealer_bjs) += 1;
    
    *(dealer_state) = ACTION_HAND_IS_OVER;
    dealer_text_message((int)(*dealer_state), 0, 0);
    
// -----------------------------------------------------------------    
  } else if ((int)(*dealer_state) == DEALER_DOES_NOT_HAVE_BLACKJACK) {
    
    // if we are here, then the user gets paid 3 to 2 (or whatever)
    *(player_bankroll) += *(bjratio)*gsl_vector_get(player_bet, 0);
    *(player_bjs) += 1;
    *(player_wins) += 1;
    
    *(dealer_state) = ACTION_HAND_IS_OVER;
    dealer_text_message((int)(*dealer_state), 0, 0);
      
  }
    

  
  return 0; 
}
