#!/bin/bash
#
# script to produce figures to understand blackjack probabilities
# using wasora's computed outputs
# 
# playing card figures are licensed under LGPL by 
# Chris Aguilar webmaster@totalnonsense.com
# http://code.google.com/p/vectorized-playing-cards/
# see cars/orig/*.txt for more information 
#
# jeremy@talador.com.ar
#

# no se como meter espacios y que el sed no se queje
text=(\
"nothing" \
"an_ace" \
"a_deuce" \
"a_three" \
"a_four" \
"a_five" \
"a_six" \
"a_seven" \
"an_eight" \
"a_nine" \
"a_face" \
"ace" \
"twelve" \
"thirteen" \
"fourteen" \
"fifteen" \
"sixteen" \
"seventeen" \
"eighteen" \
"nineteen" \
"twenty" \
)

suits=("x" "C" "S" "D" "H")
faces=("x" "J" "Q" "K")

cd data

# modify the original card figures to make them transparent
if [ -e cards/QS.png ]; then
 echo card figures already softened!
else
 echo -n softening card figures...
 op="0.2"
 cd cards
 ./clean.sh
 cp orig/*.svg .
 for i in *.svg; do
 #  echo $i
  sed -i s/-opacity:1/-opacity:${op}/g $i
  sed -i s/fill:#000000\"/fill:#000000\;fill-opacity:${op}\"/ $i
  sed -i s/fill:#000000\;display/fill:#000000\;fill-opacity:${op}\;display/ $i
  sed -i s/fill:#100f08\"/fill:#100f08\;fill-opacity:${op}\"/ $i
 # inkscape $i -D -E `basename $i .svg`.eps
 # inkscape $i -D -A `basename $i .svg`.pdf
  inkscape $i -D -d 200 -e `basename $i .svg`.png > /dev/null
 done
 cd ..
 echo ok!
fi

# compute the figures for the fixed dealer up card
echo preparing figures for fixed dealer upcard...
for i in `seq -f %02g 2 11`; do

  card=`printf %g $i`
  if [ $card -lt 10 ]; then
   file=$card
  elif [ $card -eq 11 ]; then
   file=A
  else
   file=${faces[`echo $RANDOM % 3 + 1 | bc`]}
  fi
  suit=${suits[`echo $RANDOM % 4 + 1 | bc`]}

  echo $card $file $suit

  sed    s/\$2/$i/g ../dealerup.ppl > dealerup-$i.ppl
  sed -i s/\$3/${text[${card}]}/g     dealerup-$i.ppl
  sed -i s/\$4/${file}${suit}/g       dealerup-$i.ppl
  # convertimos los underscores a espacios
  sed -i s/_/\ /g                     dealerup-$i.ppl

  pyxplot dealerup-$i.ppl
done
echo ok!

# compute the figures for the fixed player two-card total
echo preparing figures for fixed player two-card total below 12...
for i in `seq -f %02g 4 11`; do
  total=`printf %g $i`
  card1=`echo $RANDOM % \($i - 3\) + 2 | bc`
  file1=$card1
  suit1=${suits[`echo $RANDOM % 4 + 1 | bc`]}
  
  card2=`echo $total - $card1 | bc`
  file2=$card2
  suit2=${suits[`echo $RANDOM % 4 + 1 | bc`]}

  echo $total $file1 $suit1 $file2 $suit2

  sed    s/\$1/hard/g ../playerup-below.ppl   > playerup-$i.ppl
  sed -i s/\$2/$i/g                             playerup-$i.ppl
  sed -i s/\$3/${text[${total}]}/g              playerup-$i.ppl
  sed -i s/\$4/${file1}${suit1}/g               playerup-$i.ppl
  sed -i s/\$5/${file2}${suit2}/g               playerup-$i.ppl
  # convertimos los underscores a espacios
  sed -i s/_/\ /g                     playerup-$i.ppl

  pyxplot playerup-$i.ppl
done
echo ok!

# compute the figures for the fixed player two-card total
echo preparing figures for fixed player two-card total above 12...
for softhard in soft hard; do
 for i in `seq -f %02g 12 20`; do
  total=`printf %g $i`
  if [ "${softhard}" = "soft" ]; then
   card1=11
   file1=A
  else
   card1=`echo $RANDOM % 12 + $i - 10 | bc`
   if [ $card1 -lt 10 ]; then
    file1=$card1
   else
    file1=${faces[`echo $RANDOM % 3 + 1 | bc`]}
    card1=10
   fi
  fi
  suit1=${suits[`echo $RANDOM % 4 + 1 | bc`]}
  
  card2=`echo $total - $card1 | bc`
  if [ $card2 -eq 1 ]; then
   file2=A
   for j in `seq -f%02g 2 11`; do
     cp bankroll-soft-$j-22.dat bankroll-soft-$j-12.dat 
   done
  elif [ $card2 -lt 10 ]; then
   file2=$card2
  else
   file2=${faces[`echo $RANDOM % 3 + 1 | bc`]}
  fi
  suit2=${suits[`echo $RANDOM % 4 + 1 | bc`]}

  echo $softhard $total $file1 $suit1 $file2 $suit2

  sed    s/\$1/${softhard}/g ../playerup-above.ppl > playerup-${softhard}-$i.ppl
  sed -i s/\$2/$i/g                                  playerup-${softhard}-$i.ppl
  sed -i s/\$3/${text[${total}]}/g                   playerup-${softhard}-$i.ppl
  sed -i s/\$4/${file1}${suit1}/g                    playerup-${softhard}-$i.ppl
  sed -i s/\$5/${file2}${suit2}/g                    playerup-${softhard}-$i.ppl

  pyxplot playerup-${softhard}-$i.ppl
 done
done
echo ok!

cd ..
cp *.png ..
