#!/bin/bash

# el dibujito de la portada
inkscape wasorarealbook.svg -e wasorarealbook.png -C -w 550

# la version de wasora
echo "\$ wasora -v" > version.txt
wasora -v >> version.txt
echo "\$" >> version.txt

# los ejemplos
out="$PWD/examples.do.txt"

rm -f $out
echo "<?php" > sphinxsections.php
sphinxsection=0
nsections=0

for example in `echo 0*`; do

 echo $example
 cd $example

 if [ -e meta ]; then
  title=`head -n1 meta`
  author=`head -n2 meta | tail -n1`
  tags=`head -n3 meta | tail -n1`
  difficulty=`echo $example | cut -c-3`
  casedate=`bzr version-info --custom --template {date}`

  nsections=`echo ${nsections} + 1 | bc`
  sphinxsection=`echo ${sphinxsection} + 1 | bc`
  echo "\$sphinxsection[\"${example}\"] = ${sphinxsection};" >> ../sphinxsections.php
  echo "\$sphinxtitle[\"${example}\"] = \"${title}\";" >> ../sphinxsections.php


  echo "!split" >> $out
  echo "# #if FORMAT in (\"latex\", \"pdflatex\")" >> $out
  echo "\pagebreak" >> $out
  echo "# #endif" >> $out
  echo >> $out

  for i in $tags; do
    echo "idx{$i}" >> $out
  done
 
  echo >> $out
  echo "======= $title =======" >> $out
  echo >> $out
  echo "label{$example}" >> $out
  echo >> $out

  if [ -e pre.sh ]; then
    ./pre.sh
  fi

  echo >> $out
  echo "- Difficulty: $difficulty/100" >> $out
  echo "- Author: $author" >> $out
  echo "- Date: TODO" >> $out
  echo -n "- Keywords: \`$tags\`" >> $out
  echo >> $out
  echo >> $out
  

  if [ -e intro.do.txt ]; then
   echo "# #include \"$example/intro.do.txt\"" >> $out
#   cat intro.do.txt >> $out
   echo >> $out
  fi

  if [ ! -e inputs ]; then
   touch inputs
  fi

  for inputorig in `cat inputs`; do
   if [ -z "$1" ]; then
     saltear=`echo $inputorig | grep \~ | wc -l`
   else
     if [ "$example" = "`basename $1`" ]; then
       saltear=0
     else
       saltear=1
     fi
   fi
   input=`echo $inputorig | sed s/~//`
   echo "    $inputorig"

   echo >> $out
   echo "===== ${input}.was =====" >> $out
   echo >> $out

   if [ -e $input-pre.do.txt ]; then
     echo "# #include \"$example/$input-pre.do.txt\"" >> $out
#     cat $input-pre.do.txt >> $out
     echo >> $out
   fi

   echo >> $out
   echo "# #if FORMAT in (\"latex\", \"pdflatex\")" >> $out
# el chiste es que si hacemos cat el doconce rompe los comentarios
# del input de wasora porque piensa que son comentarios para el
   echo "\lstinputlisting[style=wasora,title=$input.was]{$example/$input.was}" >> $out
   echo "# #else" >> $out
   echo "!bc wasora" >> $out
   cat $input.was >> $out
   echo "!ec" >> $out
   echo "# #endif" >> $out
   echo >> $out

   if [[ $saltear -eq 0 ]]  &&  [[ -e $input.sh ]]; then
    rm -f $input.term
    i=1
    n=`wc -l < $input.sh`
    while [ $i -le $n ]; do
     cat $input.sh | head -n $i | tail -n1 > tmp
     echo -n "\$ " >> $input.term
     cat tmp >> $input.term
     chmod +x tmp
     script -aq -c ./tmp $input.term 2>&1 | grep error: > errors
     if [ ! -z "`cat errors`" ]; then
      echo "error: something happened on the way to heaven"
      cat ./tmp
      cat errors
      exit 1
     fi
     i=`echo $i + 1 | bc`
    done
    echo "\$ " >> $input.term
   fi

   echo >> $out

   if [ -e $input.term ]; then

    echo >> $out
    echo "# #if FORMAT in (\"latex\", \"pdflatex\")" >> $out
    echo "\begin{lstlisting}[style=bash]" >> $out
    echo "# #else" >> $out
    echo "!bc console" >> $out
    echo "# #endif" >> $out

    cat $input.term | grep -v Script | sed s/^\\.\$/\$/ >> $out

    echo "# #if FORMAT in (\"latex\", \"pdflatex\")" >> $out
    echo "\end{lstlisting}" >> $out
    echo "\medskip" >> $out
    echo "# #else" >> $out
    echo "!ec" >> $out
    echo "# #endif" >> $out
    echo >> $out

   fi

   echo >> $out

   if [ -e $input.figs ]; then
    cat $input.figs | while read figure; do
     cp ${figure}.png ..
#      echo "FIGURE:[$figure, width=653 height=433]" >> $out
     echo "FIGURE:[$figure]" >> $out
     echo >> $out
    done
   fi

   echo >> $out

   if [ -e $input-post.do.txt ]; then
     echo "# #include \"$example/$input-post.do.txt\"" >> $out
#     cat $input-post.do.txt >> $out
   fi

   echo >> $out

  done
 fi
 cd ..
done

echo "\$nsphinxsections = ${nsections};" >> sphinxsections.php
echo "?>">> sphinxsections.php
