/*------------ -------------- -------- --- ----- ---   --       -            -
 *  wasora assignment processing
 *
 *  Copyright (C) 2009--2013 jeremy theler
 *
 *  This file is part of wasora.
 *
 *  wasora is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  wasora is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with wasora.  If not, see <http://www.gnu.org/licenses/>.
 *------------------- ------------  ----    --------  --     -       -         -
 */

#include "wasora.h"

// un assignment puede ser tan sencillo como una variable solita o algo
// mas complejo como la asignacion de ciertos elementos de una matrix/vector
int wasora_instruction_assignment(void *arg) {
  assignment_t *assignment = (assignment_t *)arg;

  int row = 0;
  int col = 0;
  int i_min = 0;
  int i_max = 1;
  int j_min = 0;
  int j_max = 1;
  int i;
  int j;

  if (assignment->vector != NULL || assignment->matrix != NULL) {
    if (assignment->i_min.n_tokens != 0) {
      // esto es C amigos!
      i_min = (int)(round(wasora_evaluate_expression(&assignment->i_min)))-1;
      i_max = (int)(round(wasora_evaluate_expression(&assignment->i_max)));
    } else {
      i_min = assignment->row_low - 1;
      i_max = assignment->row_high;
    }
    
    if (assignment->j_min.n_tokens != 0) {
      // esto es C amigos!
      j_min = (int)(round(wasora_evaluate_expression(&assignment->j_min)))-1;
      j_max = (int)(round(wasora_evaluate_expression(&assignment->j_max)));
    } else {
      j_min = assignment->col_low - 1;
      j_max = assignment->col_high;
    }
  }
  
 
  for (i = i_min; i < i_max; i++) {
    for (j = j_min; j < j_max; j++) {

      wasora_var(wasora_special_var(i)) = (double)(i+1);
      wasora_var(wasora_special_var(j)) = (double)(j+1);

      if (assignment->row.n_tokens != 0) {
        row = (int)(wasora_evaluate_expression(&assignment->row))-1;
      } else {
        row = 0;
      }

      if (assignment->col.n_tokens != 0) {
        col = (int)(wasora_evaluate_expression(&assignment->col))-1;
      } else {
        col = 0;
      }

//      if (row >= assignment->row_low-1 && row < assignment->row_high &&
//          col >= assignment->col_low-1 && col < assignment->col_high) {
  
/* 
      if (assignment->variable != NULL) {
        wasora_push_error_message("when assigning variable '%s'", assignment->variable->name);
      } else if (assignment->vector != NULL) {
        wasora_push_error_message("when assigning row %d of vector '%s'", row+1, assignment->vector->name);
      } else {
          wasora_push_error_message("when assigning row %d and col %d of matrix '%s'", row+1, col+1, assignment->matrix->name);
      }       
*/
      wasora_assign_scalar(assignment, row, col);
//      wasora_pop_error_message(); 
//      }
      
    }
  }
  
  
  return WASORA_RUNTIME_OK;

}

// verifica si tenemos que poner los valores iniciales de una variable
void wasora_check_initial_variable(var_t *var) {
  
  if (var != NULL) {
    if (wasora_var(wasora_special_var(in_static))) {
      *var->initial_transient = wasora_value(var);
      if ((int)(wasora_var(wasora_special_var(step_static))) == 1) {
        *var->initial_static = wasora_value(var);
      }
    }
  }
  return;  
}

// verifica si tenemos que poner los valores iniciales de una variable
void wasora_check_initial_vector(vector_t *vector) {
  
  if (vector != NULL) {
    if (wasora_var(wasora_special_var(in_static))) {
      gsl_vector_memcpy(vector->initial_transient, wasora_value_ptr(vector));
      if ((int)(wasora_var(wasora_special_var(step_static))) == 1) {
        gsl_vector_memcpy(vector->initial_static, wasora_value_ptr(vector));
      }
    }
  }
  return;  
}

// verifica si tenemos que poner los valores iniciales de una variable
void wasora_check_initial_matrix(matrix_t *matrix) {
  
  if (matrix != NULL) {
    if (wasora_var(wasora_special_var(in_static))) {
      gsl_matrix_memcpy(matrix->initial_transient, wasora_value_ptr(matrix));
      if ((int)(wasora_var(wasora_special_var(step_static))) == 1) {
        gsl_matrix_memcpy(matrix->initial_static, wasora_value_ptr(matrix));
      }
    }
  }
  return;  
}


// ejecuta una asignacion escalar 
void wasora_assign_scalar(assignment_t *assignment, int row, int col) {

  int i;
  double t_min, t_max;
  int constant;
  double *current;
  double *initial_static;
  double *initial_transient;

  assignment_t *dummy;
  
  if (assignment->variable != NULL) {
    // si es un assignment sobre una variable parametrica, a comerla 
    for (i = 0; i < wasora.parametric.dimensions; i++) {
      if (assignment->variable == wasora.parametric.variable[i]) {
        return;
      }
    }
    // idem sobre un parametro a optimizar, aunque dejamos que en la primera vuelta entre
    if ((int)(wasora_var(wasora_special_var(in_outer_initial))) == 0) {
      for (i = 0; i < wasora.fit.p; i++) {
        if (assignment->variable == wasora.fit.param[i]) {
          return;
        }
      }
      for (i = 0; i < wasora.min.n; i++) {
        if (assignment->variable == wasora.min.x[i]) {
          return;
        }
      }
    }
    
    constant = assignment->variable->constant;
    current = wasora_value_ptr(assignment->variable);
    initial_static = assignment->variable->initial_static;
    initial_transient = assignment->variable->initial_transient;
    
  } else if (assignment->vector != NULL) {
    constant = assignment->vector->constant;
    current = gsl_vector_ptr(wasora_value_ptr(assignment->vector), row);
    initial_static = gsl_vector_ptr(assignment->vector->initial_static, row);
    initial_transient = gsl_vector_ptr(assignment->vector->initial_transient, row);
  } else if (assignment->matrix != NULL) {
    constant = assignment->matrix->constant;
    current = gsl_matrix_ptr(wasora_value_ptr(assignment->matrix), row, col);
    initial_static = gsl_matrix_ptr(assignment->matrix->initial_static, row, col);
    initial_transient = gsl_matrix_ptr(assignment->matrix->initial_transient, row, col);
  } else {
    return;
  }
  

  // si tenemos constant y no estamos el primer paso, a comerla
  if (constant && ((int)(wasora_var(wasora_special_var(step_static))) != 1 || (int)(wasora_var(wasora_special_var(step_transient))) != 0)) {
    return;
  }
  
  if (assignment->initial_static) {
    // si pide _init solo asignamos si estamos en static_step y en el paso uno
    if ((int)(wasora_var(wasora_special_var(in_static))) && (int)(wasora_var(wasora_special_var(step_static))) == 1) {
      *current = wasora_evaluate_expression(&assignment->expr[row+1-assignment->row_low][col+1-assignment->col_low]);
      *initial_static = *current;
      *initial_transient = *current;
    }
    
    return;
    
  } else if (assignment->initial_transient) {
    // si pide _0 solo asignamos si estamos en static_step    
    if ((int)(wasora_var(wasora_special_var(in_static)))) {
      *current = wasora_evaluate_expression(&assignment->expr[row+1-assignment->row_low][col+1-assignment->col_low]);
      *initial_transient = *current;
    }

    return;
      
  }
    
  // si el assignment pedido tiene argumento, vemos si tenemos que igualar
  if (assignment->t_min.n_tokens != 0) {
    t_min = wasora_evaluate_expression(&assignment->t_min);
    t_max = wasora_evaluate_expression(&assignment->t_max);
    if (wasora_var(wasora_special_var(time)) >= t_min && wasora_var(wasora_special_var(time)) < t_max) {
      *current = wasora_evaluate_expression(&assignment->expr[row+1-assignment->row_low][col+1-assignment->col_low]);
    }
    return;
  }

  // si no tiene argumento y resulta que hay algun assignment que si lo cumple,
  // o alguno tiene _0 o _init entonces no hacemos nada porque en algun momento va a ganar ese
  LL_FOREACH(wasora.assignments, dummy) {
    if (dummy != assignment  &&
           ( (dummy->variable != NULL && dummy->variable == assignment->variable)
           ||(dummy->vector   != NULL && dummy->vector   == assignment->vector)
           ||(dummy->matrix   != NULL && dummy->matrix   == assignment->matrix) )) {
      
      if (dummy->t_min.n_tokens != 0) {
        t_min = wasora_evaluate_expression(&dummy->t_min);
        t_max = wasora_evaluate_expression(&dummy->t_max);

        if (wasora_var(wasora_special_var(time)) >= t_min && wasora_var(wasora_special_var(time)) < t_max) {
          return;
        }
      }
  
      // si alguno tiene _0 y estamos en static no hacemos nada
      if ((int)(wasora_var(wasora_special_var(in_static))) && dummy->initial_transient) {
        return;
      }

      // idem para _init
      if ((int)(wasora_var(wasora_special_var(in_static))) && (int)(wasora_var(wasora_special_var(step_static))) == 1 && dummy->initial_static) {
        return;
      }
    }
    
  }

  // no gano ningun otro asi que hacemos la asignacion que nos pidieron
  *current = wasora_evaluate_expression(&assignment->expr[row+1-assignment->row_low][col+1-assignment->col_low]);

  if (wasora_var(wasora_special_var(in_static))) {
    *initial_transient = *current;
    if ((int)(wasora_var(wasora_special_var(step_static))) == 1) {
      *initial_static = *current;
    }
  }

  return;

}


