/*------------ -------------- -------- --- ----- ---   --       -            -
 *  wasora cleanup routines
 *
 *  Copyright (C) 2009--2013 jeremy theler
 *
 *  This file is part of wasora.
 *
 *  wasora is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  wasora is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with wasora.  If not, see <http://www.gnu.org/licenses/>.
 *------------------- ------------  ----    --------  --     -       -         -
 */

#include "wasora.h"

void wasora_polite_exit(int error) {

  if (wasora.vars != NULL) {
    wasora_var(wasora_special_var(done)) = 1;
  }

  if (wasora.ios != NULL) {
    wasora_free_shm();
  }

//  wasora_free_common();

  exit(error);
  
  

}

void wasora_free_shm(void) {
  io_t *io;
  struct semaphore_t *semaphore;
  
  LL_FOREACH(wasora.ios, io) {
    if (io->shm_pointer_double != NULL) {
      wasora_free_shared_pointer(io->shm_pointer_double, io->shm_name, io->size*sizeof(double));
    }
    if (io->shm_pointer_float != NULL) {
      wasora_free_shared_pointer(io->shm_pointer_float, io->shm_name, io->size*sizeof(float));
    }
    if (io->shm_pointer_char != NULL) {
      wasora_free_shared_pointer(io->shm_pointer_char, io->shm_name, io->size*sizeof(char));
    }
    
    if (io->shm_name != NULL) {
      free(io->shm_name);
    }
  }
  
  LL_FOREACH(wasora.semaphores, semaphore) {
    wasora_free_semaphore(semaphore->pointer, semaphore->name);
  }
  
  return;
}

void wasora_destroy_expression(expr_t *expr, int index) {
  
  int i, j;
  int nallocs = 0;
  
  for (i = 0; i < expr->n_tokens; i++) {
    if (expr->token[i].arg != NULL) {
      switch (expr->token[i].type) {
        case EXPR_BUILTIN_FUNCTION:
          nallocs = expr->token[i].builtin_function->max_arguments;
        break;
        case EXPR_BUILTIN_FUNCTIONAL: 
          nallocs = expr->token[i].builtin_functional->max_arguments;
        break;
        case EXPR_FUNCTION:
          nallocs = expr->token[i].function->n_arguments;
        break;
      }
      for (j = nallocs-1; j >= 0; j--) {
        wasora_destroy_expression(&expr->token[i].arg[j], 0);
      }
      free(expr->token[i].arg);
    }
    if (expr->token[i].vector != NULL) {
      free(expr->token[i].vector_arg);
    }
    if (expr->token[i].aux != NULL) {
      free(expr->token[i].aux);
    }
  }
  
  // solamente tenemos que hacer free de esto si somos el primer elemento de un arreglo
  if (expr->n_tokens != 0 && index == 0) {
    free(expr->token);
  }
  
  if (expr->string != NULL) {
    free(expr->string);
    expr->string = NULL;
  }
  
  
  return;
  
}

/* TODO: hay que liberar un monton de cosas mas! */
void wasora_finalize(void) {

  int i, j;
//  int index_i, index_j;

  for (i = 0; i < wasora.i_plugin; i++) {
    wasora.plugin[i].finalize();
  }
  
  if (wasora.min.n != 0) {
    if (wasora.min.step != NULL) {
      for (j = wasora.min.n_steps-1; j >= 0; j--) {
        wasora_destroy_expression(&wasora.min.step[j], 0);
      }
      free(wasora.min.step);
    }
    if (wasora.min.guess != NULL) {
      for (j = wasora.min.n-1; j >= 0; j--) {
        wasora_destroy_expression(&wasora.min.guess[j], 0);
      }
      free(wasora.min.guess);
    }
    if (wasora.min.gradient != NULL) {
      for (j = wasora.min.n-1; j >= 0; j--) {
        wasora_destroy_expression(&wasora.min.gradient[j], 0);
      }
      free(wasora.min.gradient);
    }
    
    wasora_destroy_expression(&wasora.min.gradtol, 0);
    wasora_destroy_expression(&wasora.min.tol, 0);
    
    free(wasora.min.x);
  }
  
  if (wasora.fit.p != 0) {
    if (wasora.fit.guess != NULL) {
      for (j = wasora.fit.p-1; j >= 0; j--) {
        wasora_destroy_expression(&wasora.fit.guess[j], 0);
      }
      free(wasora.fit.guess);
    }

    wasora_destroy_expression(&wasora.fit.deltaepsabs, 0);
    wasora_destroy_expression(&wasora.fit.deltaepsrel, 0);

    free(wasora.fit.param);
    free(wasora.fit.sigma);
  }
          

/*  
  if (wasora_parser.i_outputfile != 0) {
    for (i = 0; i < wasora_parser.i_outputfile; i++) {
      if (wasora.first.outputfile[i].name != NULL) {
        free(wasora.first.outputfile[i].name);
      }
      if (wasora.first.outputfile[i].path != NULL) {
        free(wasora.first.outputfile[i].path);
      }
      if (wasora.first.outputfile[i].pointer != NULL) {
        fclose(wasora.first.outputfile[i].pointer);
      }
    }
    free(wasora.first.outputfile);
  }
*/
/*  
  if (wasora_parser.i_print_vector != 0) {
    for (i = 0; i < wasora_parser.i_print_vector; i++) {
      wasora_destroy_expression(&wasora.first.print_vector[i].condition, 0);
      free(wasora.first.print_vector[i].separator);
      free(wasora.first.print_vector[i].format);
      free(wasora.first.print_vector[i].vector);
    }
    free(wasora.first.print_vector);
  }
*/
/*
  if (wasora_parser.i_print != 0) {
    for (i = 0; i < wasora_parser.i_print; i++) {
      free(wasora.first.print[i].separator);
      for (j = wasora.first.print[i].n_things-1; j >= 0 ; j--) {
        if (wasora.first.print[i].text != NULL) {
          free(wasora.first.print[i].text[j]);
        }
        if (wasora.first.print[i].format != NULL) {
          free(wasora.first.print[i].format[j]);
        }
        wasora_destroy_expression(&wasora.first.print[i].expression[j], 0);
      }
      free(wasora.first.print[i].text);
      free(wasora.first.print[i].format);
      free(wasora.first.print[i].what);
      free(wasora.first.print[i].expression);

      if (wasora.first.print[i].condition.n_tokens != 0) {
        wasora_destroy_expression(&wasora.first.print[i].condition, 0);
      }
    }

    free(wasora.first.print);
  }
*/
/*  
  if (wasora_parser.i_print_function != 0) {
    for (i = 0; i < wasora_parser.i_print_function; i++) {
      if (wasora.first.print_function[i].min != NULL) {
        for (j = 0; j < wasora.first.print_function[i].function[0]->n_arguments; j++) {
          wasora_destroy_expression(&wasora.first.print_function[i].min[j], 0);
        }
        free(wasora.first.print_function[i].min);
      }
      if (wasora.first.print_function[i].max != NULL) {
        for (j = 0; j < wasora.first.print_function[i].function[0]->n_arguments; j++) {
          wasora_destroy_expression(&wasora.first.print_function[i].max[j], 0);
        }
        free(wasora.first.print_function[i].max);
      }
      if (wasora.first.print_function[i].step != NULL) {
        for (j = 0; j < wasora.first.print_function[i].function[0]->n_arguments; j++) {
          wasora_destroy_expression(&wasora.first.print_function[i].step[j], 0);
        }
        free(wasora.first.print_function[i].step);        
      }
      

      if (wasora.first.print_function[i].condition.n_tokens != 0) {
        wasora_destroy_expression(&wasora.first.print_function[i].condition, 0);
      }
      
      free(wasora.first.print_function[i].separator);
      free(wasora.first.print_function[i].format);
      free(wasora.first.print_function[i].function);
    }
    
    free(wasora.first.print_function);
  }
*/
/*  
  if (wasora_parser.i_assignment != 0) {
    for (i = 0; i < wasora_parser.i_assignment; i++) {
      for (index_i = wasora.first.assignment[i].size_i-1; index_i >= 0; index_i--) {
        for (index_j = wasora.first.assignment[i].size_j-1; index_j >= 0; index_j--) {
          wasora_destroy_expression(&wasora.first.assignment[i].algebraic_expr[index_i][index_j], index_j);
        }
        free(wasora.first.assignment[i].algebraic_expr[index_i]);
      }
      free(wasora.first.assignment[i].algebraic_expr);
      wasora_destroy_expression(&wasora.first.assignment[i].index_i, 0);
      wasora_destroy_expression(&wasora.first.assignment[i].index_j, 0);
    
    }
    free(wasora.first.assignment);
  }
*/

/*  
  if (wasora_parser.i_function != 0) {
    for (i = 0; i < wasora_parser.i_function; i++) {

      if (wasora.first.function[i].arg_name != NULL) {
        for (j = wasora.first.function[i].n_arguments-1; j >= 0 ; j--) {
          if (wasora.first.function[i].arg_name[j] != NULL) {
            free(wasora.first.function[i].arg_name[j]);
          }
        }
        free(wasora.first.function[i].arg_name);
      }
      
      if (wasora.first.function[i].var_argument != NULL) {
        free(wasora.first.function[i].var_argument);
      }
      
      if (wasora.first.function[i].column != NULL) {
        free(wasora.first.function[i].column);
      }
      
      if (wasora.first.function[i].vector_argument != NULL) {
        free(wasora.first.function[i].vector_argument);
      }
      
      if (wasora.first.function[i].data_argument != NULL) {
        for (j = 0; j < wasora.first.function[i].n_arguments; j++) {
          free(wasora.first.function[i].data_argument[j]);
        }
        free(wasora.first.function[i].data_argument);
      }
      if (wasora.first.function[i].data_value != NULL) {
        free(wasora.first.function[i].data_value);
      }

      if (wasora.first.function[i].interp != NULL) {
        gsl_interp_free(wasora.first.function[i].interp);
      }
      
      if (wasora.first.function[i].interp_accel != NULL) {
        gsl_interp_accel_free(wasora.first.function[i].interp_accel);
      }
      
      if (wasora.first.function[i].kd != NULL) {
        kd_free(wasora.first.function[i].kd);
      }
      
      
      
      if (wasora.first.function[i].name != NULL) {
        free(wasora.first.function[i].name);
      }
      
      if (wasora.first.function[i].data_file != NULL) {
        free(wasora.first.function[i].data_file);
      }
      
      wasora_destroy_expression(&wasora.first.function[i].algebraic_expression, 0);

    }
    free(wasora.first.function);
  }
*/  
/*  
  if (wasora_parser.i_matrix != 0) {
    for (i = 0; i < wasora_parser.i_matrix; i++) {
      if (wasora.first.matrix[i].name != NULL) {
        free(wasora.first.matrix[i].name);
      }
    }
    free(wasora.first.matrix);
  }
*/
/*  
  if (wasora_parser.i_vector != 0) {
    for (i = 0; i < wasora_parser.i_vector; i++) {
      if (wasora.first.vector[i].element != NULL) {
        free(wasora.first.vector[i].element);
      }
      if (wasora.first.vector[i].name != NULL) {
        free(wasora.first.vector[i].name);
      }
    }
    free(wasora.first.vector);
  }
*/
/*  
  if (wasora_parser.i_number != 0) {
    for (i = 0; i < wasora_parser.i_number; i++) {
      if (wasora.first.number[i].name != NULL) {
        free(wasora.first.number[i].name);
      }
    }
    free(wasora.first.number);
  }
 */

/*  
  if (wasora_parser.i_var != 0) {
    for (i = 0; i < wasora_parser.i_var; i++) {
      if (wasora.first.var[i].name != NULL) {
        free(wasora.first.var[i].name);
      }
    }
    free(wasora.first.var);
  }
*/
  
/*  
  if (wasora_parser.i_instruction != 0) {
    free(wasora.first.instruction);
  }
*/  

  return;

}
