/*------------ -------------- -------- --- ----- ---   --       -            -
 *  wasora template replacement routines
 *
 *  Copyright (C) 2012--2014 jeremy theler
 *
 *  This file is part of wasora.
 *
 *  wasora is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  wasora is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with wasora.  If not, see <http://www.gnu.org/licenses/>.
 *------------------- ------------  ----    --------  --     -       -         -
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>


#include "wasora.h"


int wasora_instruction_templ(void *arg) {
  templ_t *templ = (templ_t *)arg;
  print_token_t *print_token;
  char *current_format = (templ->print_tokens != NULL)?templ->print_tokens->format:NULL;
  
  FILE *input;
  FILE *output;
  int c, c2;
  int n;
  int found;
  
  if ((input = wasora_fopen(templ->filename_input, "r")) == NULL) {
    wasora_push_error_message("cannot open file '%s' for reading: %s", templ->filename_input, strerror(errno));
    return WASORA_RUNTIME_ERROR;
  }

  if ((output = wasora_fopen(templ->filename_output, "w")) == NULL) {
    wasora_push_error_message("cannot open file '%s' for writing: %s", templ->filename_output, strerror(errno));
    return WASORA_RUNTIME_ERROR;
  }
  
  while ((c = fgetc(input)) != EOF) {
    
    if (c == templ->wildcard && isdigit(c2 = fgetc(input))) {
      
      ungetc(c2, input);
      if (fscanf(input, "%d", &n) != 1) {
        wasora_push_error_message("out of data in file '%s'", templ->filename_input);
        return WASORA_RUNTIME_ERROR;
      }
      if (n == 0) {
        wasora_push_error_message("template file '%s' contains wildcard '%c' plus zero", templ->filename_input, templ->wildcard);
        return WASORA_PARSER_ERROR;
      }
      
      found = 0;
      LL_FOREACH(templ->print_tokens, print_token) {
        if (print_token->format != NULL) {
          current_format = print_token->format;
          
        } else if (print_token->expression.n_tokens != 0 && print_token->index == n-1) {
          found = 1;
          fprintf(output, (current_format!=NULL)?current_format:"%g", wasora_evaluate_expression(&print_token->expression));
        }
      }
      
      if (!found) {
        wasora_push_error_message("template file '%s' asks for argument %d", templ->filename_input, n);
        return WASORA_PARSER_ERROR;
      }
      
    } else {
      
      fputc(c, output);
      if (c == templ->wildcard) {
        fputc(c2, output);
      }
          
    }
    
  }
  
  
  
  fclose(output);
  fclose(input);
  
  return 0;
  
}
